@@ -13,17 +13,12 @@ def main() -> None:
         sys.argv.append("spec")
 
     try:
-        with open(config.scriptPath("..", "semver.txt"), encoding="utf-8") as fh:
+        with open(config.scriptPath("semver.txt"), encoding="utf-8") as fh:
             semver = fh.read().strip()
             semverText = f"Bikeshed v{semver}: "
     except FileNotFoundError:
-        try:
-            with open(config.scriptPath("semver.txt"), encoding="utf-8") as fh:
-                semver = fh.read().strip()
-                semverText = f"Bikeshed v{semver}: "
-        except FileNotFoundError:
-            semver = "???"
-            semverText = ""
+        semver = "???"
+        semverText = ""
 
     argparser = argparse.ArgumentParser(description=f"{semverText}Processes spec source files into valid HTML.")
     argparser.add_argument("--version", action="version", version=semver)
@@ -578,7 +573,7 @@ def handleDebug(options: argparse.Namespace, extras: list[str]) -> None:
 
 
 def handleRefs(options: argparse.Namespace, extras: list[str]) -> None:
-    from . import metadata
+    from . import metadata, datablocks
     from .refs import ReferenceManager
     from .Spec import Spec
 
@@ -591,7 +586,7 @@ def handleRefs(options: argparse.Namespace, extras: list[str]) -> None:
         rm = doc.refs
     else:
         rm = ReferenceManager()
-        rm.initializeRefs()
+        rm.initializeRefs(datablocks=datablocks)
     if options.text:
         options.text = options.text
     refs = rm.queryAllRefs(
