@@ -1,7 +1,9 @@
-from .. import h, messages as m
+from __future__ import annotations
+
+from .. import h, messages as m, t
 
 
-def printjson(x, indent=2, level=0):
+def printjson(x: t.Any, indent: str | int = 2, level: int = 0) -> str:
     if isinstance(indent, int):
         # Convert into a number of spaces.
         indent = " " * indent
@@ -21,14 +23,14 @@ def printjson(x, indent=2, level=0):
     # return json.dumps(obj, indent=2, default=lambda x:x.__json__())
 
 
-def getjson(x):
+def getjson(x: t.Any) -> t.Any:
     try:
         return x.__json__()
     except AttributeError:
         return x
 
 
-def printjsonobject(x, indent, level):
+def printjsonobject(x: t.Any, indent: str, level: int) -> str:
     x = getjson(x)
     ret = ""
     maxKeyLength = 0
@@ -44,7 +46,7 @@ def printjsonobject(x, indent, level):
     return ret
 
 
-def printjsonobjectarray(x, indent, level):
+def printjsonobjectarray(x: list[dict[str, t.Any]], indent: str, level: int) -> str:
     # Prints an array of objects
     x = getjson(x)
     ret = ""
@@ -55,7 +57,7 @@ def printjsonobjectarray(x, indent, level):
     return ret
 
 
-def printjsonsimplearray(x, indent, level):  # pylint: disable=unused-argument
+def printjsonsimplearray(x: list, indent: str, level: int) -> str:  # pylint: disable=unused-argument
     x = getjson(x)
     ret = m.printColor("[", "blue")
     for i, v in enumerate(x):
@@ -66,7 +68,7 @@ def printjsonsimplearray(x, indent, level):  # pylint: disable=unused-argument
     return ret
 
 
-def printjsonprimitive(x):
+def printjsonprimitive(x: t.Any) -> str:
     x = getjson(x)
     if isinstance(x, int):
         return str(x)
